#include "config.h"
#include <stdlib.h>
#include <string.h>

#ifdef OS_LINUX
	#include <sys/socket.h>
	#include <netinet/in.h>
#endif

#ifdef OS_WINDOWS
	#include <winsock.h>
#endif

#include "ycht_pkt.h"

/* sets */
int pktSetLen(char * pkt, short unsigned int data_len)
{
	short unsigned int *ptr;
	ptr=(short unsigned int *)(pkt+14);
	*ptr=htons(data_len);
	return 0;
}

int pktSetProtocol(char * pkt, char * prot)
{
	memcpy(pkt,prot,4);
	return 0;
}

int pktSetType(char * pkt, const char * type)
{
	memcpy(pkt+8,type,4);
	return 0;
}

int pktSetUndefined(char * pkt)
{
	const char undef[]={0,0,1,0};
	memcpy(pkt+4,undef,4);
	return 0;
}

/* get's */
short unsigned int pktGetLen(char * pkt)
{
	short unsigned int *ptr;
	ptr=(short unsigned int *)(pkt+14);
	return ntohs(*ptr);
}

int pktTotalLen(char * pkt)
{
	return HDR_LEN+pktGetLen(pkt);
}

int pktGetProt(char * pkt)
{
	if(!memcmp(pkt,"YCHT",4))
		return PROT_YCHT;
	else if(!memcmp(pkt,"YMSG",4))
		return PROT_YMSG;
	else
		return PROT_UNDF;
}

/* get data items from a packet */
int pktGetDataItem(unsigned char * pkt, char * item, int offset)
{
	unsigned char *ptr;
	int length,current_offset;

	for(current_offset=0,length=0,ptr=(pkt+HDR_LEN);;length++){
		if(ptr+length>=pkt+pktTotalLen(pkt) || *(ptr+length)==0xC0){
			if(current_offset==offset)
			{
				strncpy(item,ptr,length);
				item[length]='\0';
				break;
			}
			else
			{
				ptr=strchr(ptr,0x80)+1;
				length=0;
				current_offset++;
			}
		}
	}
	return 0;
}

int pktGetType(const char * pkt)
{
	/* somehow use case instead? */
	if(!memcmp(pkt+8,YCHT_TALK,4))
		return PKT_TALK;
	else if(!memcmp(pkt+8,YCHT_JOIN,4))
		return PKT_JOIN;
	else if(!memcmp(pkt+8,YCHT_LEAVE,4))
		return PKT_LEAVE;
	else if(!memcmp(pkt+8,YCHT_AWAY,4))
		return PKT_AWAY;
	else
		return PKT_UNDEF;

	return 0;
}

/* packet creation */
int SendYCHTPkt(int fd,const char *type,struct ItemNode **list)
{
	struct ItemNode *current, *prev;
	int length=0;
	char item_number=1;
	char pkt[1024];

	// create the header
	pktSetProtocol(pkt,"YCHT");
	pktSetUndefined(pkt);
	pktSetType(pkt,type);
	memcpy(pkt+12,"\x00\x00",2);		/* padding */
	pkt[16]='\0';										/* this is for the strcat'ing we do later on */

	/* if no list was passed (ie, ping packet, no payload) set len to 0, exit */
	if(list==NULL)
	{
		pktSetLen(pkt,0);
		send(fd,pkt,16,0);
		return 0;
	}

	current=*list;
	length=0;

	/* get the item, then destroy it from the list */
	while((prev=current))
	{
		/* first item doesn't have padding before it */
		if(item_number++ > 1)
		{
			strcat(pkt+16,"\x01");
			length++;
		}
		strcat(pkt+16,current->item);
		length+=strlen(current->item);
		/* advance current, then delete the previous one */
		current=current->next;
		free(prev);
	}
	*list=NULL;

	/* finally, set the packets length and send */
	pktSetLen(pkt,length);
	send(fd,pkt,pktTotalLen(pkt),0); /* TODO handle partial sends */

	return 0;
}

void ItemAdd(struct ItemNode **list, char * item)
{
	struct ItemNode *newnode,*current;

	newnode=(struct ItemNode *) malloc(sizeof(struct ItemNode));
	strncpy(newnode->item,item,1024);
	newnode->next=NULL;

	current=*list;

	if(current==NULL)
	{
		*list=newnode;
	}
	else
	{
		while(current->next!=NULL) current=current->next;
			current->next=newnode;
	}
}        
